@@ -1,6 +1,6 @@
 ;;; navi2ch-history.el --- global history module for navi2ch
 
-;; Copyright (C) 2001, 2002 by Navi2ch Project
+;; Copyright (C) 2001-2003 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -26,8 +26,8 @@
 
 ;;; Code:
 (provide 'navi2ch-history)
-(defvar navi2ch-history-ident
-  "$Id: navi2ch-history.el,v 1.17 2002/09/25 09:55:43 mami Exp $")
+(defconst navi2ch-history-ident
+  "$Id: navi2ch-history.el,v 1.21 2004/05/02 14:41:51 nanashi Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -157,6 +157,7 @@ key $B$O(B (concat uri artid)$B!#(B
   (setq major-mode 'navi2ch-history-mode)
   (setq mode-name "Navi2ch History")
   (setq buffer-read-only t)
+  (buffer-disable-undo)
   (use-local-map navi2ch-history-mode-map)
   (navi2ch-history-setup-menu)
   (run-hooks 'navi2ch-bm-mode-hook 'navi2ch-history-mode-hook))
@@ -174,8 +175,7 @@ key $B$O(B (concat uri artid)$B!#(B
 			  (assq 'subject article)
 			  (assq 'artid article)))))
 	       navi2ch-history-alist)))
-    (if info
-	(navi2ch-save-info navi2ch-history-file info t))))
+    (navi2ch-save-info navi2ch-history-file info t)))
 
 (defun navi2ch-history-load-info ()
   (setq navi2ch-history-alist
