@@ -1,6 +1,6 @@
 ;;; navi2ch-list.el --- board list module for navi2ch
 
-;; Copyright (C) 2000-2003 by Navi2ch Project
+;; Copyright (C) 2000-2004 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -26,8 +26,8 @@
 
 ;;; Code:
 (provide 'navi2ch-list)
-(defvar navi2ch-list-ident
-  "$Id: navi2ch-list.el,v 1.61 2003/07/16 12:30:48 mami Exp $")
+(defconst navi2ch-list-ident
+  "$Id: navi2ch-list.el,v 1.71 2004/05/02 15:12:50 nanashi Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -42,8 +42,7 @@
     (define-key map "z" 'navi2ch-suspend)
     (define-key map "s" 'navi2ch-list-sync)
     (define-key map " " 'navi2ch-list-select-current-board)
-    (define-key map [del] 'scroll-down)
-    (define-key map [backspace] 'scroll-down)
+    (navi2ch-define-delete-keys map 'scroll-down)
     (define-key map "a" 'navi2ch-list-add-bookmark)
     (define-key map "b" 'navi2ch-list-toggle-bookmark)
     (navi2ch-define-mouse-key map 2 'navi2ch-list-mouse-select)
@@ -69,7 +68,7 @@
   "Menu definition for navi2ch-list.")
 
 (defvar navi2ch-list-ignore-category-list
-  '("$B%A%c%C%H(B" "$B$*3($+$-(B" "$B1?1D(B" "$B%D!<%kN`(B" "$BB>$N7G<(HD(B" "$BB>$N%5%$%H(B"))
+  '("$B%A%c%C%H(B" "$B$*3($+$-(B" "$B1?1D0FFb(B" "$B%D!<%kN`(B" "$BB>$N7G<(HD(B" "$BB>$N%5%$%H(B"))
 (defvar navi2ch-list-buffer-name "*navi2ch list*")
 (defvar navi2ch-list-current-list nil)
 (defvar navi2ch-list-category-list nil)
@@ -126,10 +125,10 @@
 
 (defun navi2ch-list-get-global-bookmark-board-list ()
   (mapcar (lambda (x)
-	     (list (cons 'name (cadr x))
-		   (cons 'type 'bookmark)
-		   (cons 'id (car x))))
-	   navi2ch-bookmark-list))
+	    (list (cons 'name (cadr x))
+		  (cons 'type 'bookmark)
+		  (cons 'id (car x))))
+	  navi2ch-bookmark-list))
 
 (defun navi2ch-list-get-global-bookmark-category ()
   (navi2ch-list-get-category
@@ -254,18 +253,15 @@
 
 (defun navi2ch-list-insert-bookmarks (list)
   (let ((bookmark (cdr (assq 'bookmark navi2ch-list-current-list)))
-	list2)
+	alist)
     (dolist (x (navi2ch-list-get-board-name-list list))
       (let ((node (navi2ch-list-bookmark-node x)))
 	(when (member node bookmark)
 	  ;; $B%j%9%H$N8e$K$"$k%N!<%I$rM%@h(B
-	  (let ((found (catch 'break
-			 (dolist (y list2)
-			   (if (equal (navi2ch-list-bookmark-node y) node)
-			       (throw 'break y))))))
-	    (if found (setq list2 (delete found list2))))
-	  (setq list2 (cons x list2)))))
-    (navi2ch-list-insert-board-names-subr (nreverse list2))))
+	  (setq alist (delq (assoc node alist) alist))
+	  (push (cons node x) alist))))
+    (navi2ch-list-insert-board-names-subr (mapcar #'cdr
+						  (nreverse alist)))))
 
 (defun navi2ch-list-toggle-open ()
   "$B%+%F%4%j$r3+$$$?$jJD$8$?$j$9$k!#(B"
@@ -463,21 +459,21 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
   (message "applying board changes...")
   (let ((added-list (cdr (assq 'add changed-status)))
 	(changed-list (cdr (assq 'change changed-status))))
-  (when changed-status
-    (setq navi2ch-list-current-list
-	  (navi2ch-put-alist 'change
-			     (append (mapcar (lambda (id)
-					       (cons id 'add))
-					     added-list)
-				     (mapcar (lambda (pair)
-					       (cons (car pair) 'change))
-					     changed-list))
-			     navi2ch-list-current-list))
-    (navi2ch-change-log-directory changed-list)
-    (navi2ch-bookmark-change changed-list)
-    (navi2ch-history-change changed-list)
-    (navi2ch-list-change changed-list)
-    (message "applying board changes...done"))))
+    (when changed-status
+      (setq navi2ch-list-current-list
+	    (navi2ch-put-alist 'change
+			       (append (mapcar (lambda (id)
+						 (cons id 'add))
+					       added-list)
+				       (mapcar (lambda (pair)
+						 (cons (car pair) 'change))
+					       changed-list))
+			       navi2ch-list-current-list))
+      (navi2ch-change-log-directory changed-list)
+      (navi2ch-bookmark-change changed-list)
+      (navi2ch-history-change changed-list)
+      (navi2ch-list-change changed-list)
+      (message "applying board changes...done"))))
 
 (defun navi2ch-list-get-changed-category (category-list)
   (let ((alist (navi2ch-alist-list-to-alist
@@ -581,6 +577,10 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
 		(when (and (not ignore)
 			   (string-match "href=\\(.+/\\([^/]+\\)/\\)" attr)
 			   (setq url (match-string 1 attr))
+			   (setq url (or (cdr (assoc
+					       url
+					       navi2ch-list-moved-board-alist))
+					 url))
 			   (setq id (navi2ch-list-board-id-from-url url))
 			   (navi2ch-list-valid-board url))
 		  (when (and (setq u (cdr (assoc id id-to-url-alist)))
@@ -798,10 +798,20 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
   (navi2ch-search-cache-subr
    (navi2ch-list-get-current-category-list)))
 
+(defun navi2ch-list-search-current-board-orphan ()
+  (interactive)
+  (navi2ch-search-orphan-subr (list (get-text-property (point) 'board))))
+
+(defun navi2ch-list-search-current-category-orphan ()
+  (interactive)
+  (navi2ch-search-orphan-subr
+   (navi2ch-list-get-current-category-list)))
+
 (defun navi2ch-list-search ()
   (interactive)
   (let ((ch (navi2ch-read-char-with-retry
-	     "Search for: s)ubject a)rticle c)ache: " nil '(?s ?a ?c)))
+	     "Search for: s)ubject a)rticle c)ache o)rphan: "
+	     nil '(?s ?a ?c ?o)))
 	(ch2 (if (get-text-property (point) 'board)
 		 (navi2ch-read-char-with-retry
 		  "Search from: b)oard c)ategory a)ll: " nil '(?b ?c ?a))
@@ -818,7 +828,11 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
 	  ((eq ch ?c)
 	   (cond ((eq ch2 ?b) (navi2ch-list-search-current-board-cache))
 		 ((eq ch2 ?c) (navi2ch-list-search-current-category-cache))
-		 ((eq ch2 ?a) (navi2ch-search-all-cache)))))))
+		 ((eq ch2 ?a) (navi2ch-search-all-cache))))
+	  ((eq ch ?o)
+	   (cond ((eq ch2 ?b) (navi2ch-list-search-current-board-orphan))
+		 ((eq ch2 ?c) (navi2ch-list-search-current-category-orphan))
+		 ((eq ch2 ?a) (navi2ch-search-all-orphan)))))))
 
 ;;; expire
 (defun navi2ch-list-expire-current-board (&optional ask)
@@ -876,5 +890,9 @@ changed-list $B$O(B '((board-id old-board new-board) ...) $B$J(B alist$B!#
 	      (kill-new str)
 	      (message "copy: %s" str))))))))
 
+(defun navi2ch-list-url-at-point (point)
+  (let ((board (get-text-property point 'board)))
+    (cdr (assq 'uri board))))
+
 (run-hooks 'navi2ch-list-load-hook)
 ;;; navi2ch-list.el ends here
