@@ -1,6 +1,6 @@
 ;;; navi2ch.el --- Navigator for 2ch for Emacsen
 
-;; Copyright (C) 2000-2003 by Navi2ch Project
+;; Copyright (C) 2000-2004 by Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -22,8 +22,8 @@
 
 ;;; Code:
 (provide 'navi2ch)
-(defvar navi2ch-ident
-  "$Id: navi2ch.el,v 1.62 2003/09/05 22:15:16 mizmiz Exp $")
+(defconst navi2ch-ident
+  "$Id: navi2ch.el,v 1.72 2004/05/02 14:41:52 nanashi Exp $")
 
 (eval-when-compile (require 'cl))
 
@@ -84,12 +84,14 @@
       (load-file navi2ch-update-file))
     (load navi2ch-init-file t)
     (navi2ch-lock)
-    (if navi2ch-auto-update
-	(condition-case err
-	    (navi2ch-update)
-	  ((error quit)
-	   (navi2ch-unlock)
-	   (signal (car err) (cdr err)))))
+    (when navi2ch-auto-update
+      (let ((done nil))
+	(unwind-protect
+	    (progn
+	      (navi2ch-update)
+	      (setq done t))
+	  (unless done
+	    (navi2ch-unlock)))))
     (add-hook 'kill-emacs-hook 'navi2ch-kill-emacs-hook)
     (run-hooks 'navi2ch-load-status-hook)
     (run-hooks 'navi2ch-hook)
@@ -314,14 +316,23 @@ BACKUP $B$,(B non-nil $B$N>l9g$O85$N%U%!%$%k$r%P%C%/%"%C%W$9$k!#(B"
 		   (not (file-regular-p file))
 		   (file-newer-than-file-p backup-file file))
 	       (yes-or-no-p
-		"$BLdBjH/@8!#%P%C%/%"%C%W%U%!%$%k$+$iFI$_9~$_$^$9$+(B? "))
+		(format
+		 "%s $B$NFI$_9~$_$GLdBjH/@8!#%P%C%/%"%C%W%U%!%$%k$+$iFI$_9~$_$^$9$+(B? "
+		 file)))
       (setq file backup-file)))
   (when (file-regular-p file)
     (let ((coding-system-for-read navi2ch-coding-system))
-      (with-temp-buffer
-	(insert-file-contents file)
-	(let ((standard-input (current-buffer)))
-	  (read))))))
+      (navi2ch-ifemacsce
+	  (with-temp-buffer
+	    (insert-file-contents file)
+	    (goto-char (point-min))
+	    (while (search-forward "..." nil t)
+	      (replace-match ""))
+	    (car (read-from-string (buffer-string))))
+	(with-temp-buffer
+	  (insert-file-contents file)
+	  (let ((standard-input (current-buffer)))
+	    (read)))))))
 
 (defun navi2ch-split-window (display)
   "window $B$rJ,3d$9$k!#(B
@@ -352,7 +363,9 @@ DISPLAY $B$,(B `article' $B$N$H$-$O(B article $B$rI=<($9$kMQ$KJ,3d$9$k!#(B
 
 (defun navi2ch-goto-url (url &optional force)
   "URL $B$+$i%9%l$^$?$OHD$rA*$V(B"
-  (interactive "sURL: ")
+  (interactive (list (navi2ch-read-string "URL: "
+					  (get-text-property (point) 'url))
+		     current-prefix-arg))
   (let ((article (navi2ch-article-url-to-article url))
 	(board (navi2ch-board-url-to-board url)))
     (cond (article
@@ -414,12 +427,23 @@ CHANGED-LIST $B$K$D$$$F$O(B `navi2ch-list-get-changed-status' $B$r;2>H!#(B"
 (defun navi2ch-update ()
   "navi2ch-update.el $B$r%@%&%s%m!<%I$7$F<B9T$9$k!#(B"
   (interactive)
-  (let ((new (concat navi2ch-update-file ".new")))
+  (let ((new (concat navi2ch-update-file ".new"))
+	(time (and (file-exists-p navi2ch-update-file)
+		   (nth 5 (file-attributes navi2ch-update-file))))
+	(asc (concat navi2ch-update-file ".asc"))
+	(asc-url (concat navi2ch-update-url ".asc"))
+	verified)
     (when (and navi2ch-update-url
 	       (not (string= navi2ch-update-url ""))
 	       (not navi2ch-offline)
-	       (navi2ch-net-update-file navi2ch-update-url new)
+	       (navi2ch-net-update-file navi2ch-update-url new time)
 	       (file-exists-p new)
+	       (if navi2ch-pgp-verify-command-line
+		   (and (navi2ch-net-update-file asc-url asc 'file)
+			(file-exists-p asc)
+			(navi2ch-verify-signature-file asc new)
+			(setq verified t))
+		 t)
 	       (or (not (file-exists-p navi2ch-update-file))
 		   (not (= (nth 7 (file-attributes navi2ch-update-file))
 			   (nth 7 (file-attributes new))))
@@ -431,7 +455,9 @@ CHANGED-LIST $B$K$D$$$F$O(B `navi2ch-list-get-changed-status' $B$r;2>H!#(B"
 			   (insert-file-contents-literally new)
 			   (buffer-string)))))
 	       (yes-or-no-p
-		"navi2ch-update.el$B$,99?7$5$l$^$7$?!#J]B8$7$F<B9T$7$^$9$+(B? "))
+		(concat "navi2ch-update.el $B$,99?7$5$l$^$7$?(B"
+			(unless verified "($BL$8!>Z(B)")
+			"$B!#J]B8$7$F<B9T$7$^$9$+(B? ")))
       (navi2ch-rename-file new navi2ch-update-file t)
       (load navi2ch-update-file))
     (if (file-exists-p new)
@@ -486,6 +512,7 @@ CHANGED-LIST $B$K$D$$$F$O(B `navi2ch-list-get-changed-status' $B$r;2>H!#(B"
       (with-output-to-temp-buffer "*Navi2ch Ident List*"
 	(princ (mapconcat 'symbol-value ident-list "\n"))))))
 
+
 (eval-when-compile
   (autoload 'browse-url-interactive-arg "browse-url"))
 
@@ -502,6 +529,35 @@ CHANGED-LIST $B$K$D$$$F$O(B `navi2ch-list-get-changed-status' $B$r;2>H!#(B"
     (message "falling back...")
     (apply 'navi2ch-browse-url-internal url args)))
 
+(defun navi2ch-url-at-point (point)
+  "POINT $B$N2<$N%j%s%/$r;X$9(B URL $B$rF@$k!#(B
+\(defadvice browse-url-url-at-point
+  (around my-browse-url-url-at-point-navi2ch activate compile)
+  (let ((url (navi2ch-url-at-point (point))))
+    (if url
+	(setq ad-return-value url)
+      ad-do-it)))
+$B$N$h$&$K$9$k$H!"%j%s%/$KBP$7$F(B browse-url $B$r%$%s%?%i%/%F%#%V$K(B
+$B<B9T$G$-$k!#(B"
+  (let ((alist `((navi2ch-list-mode . navi2ch-list-url-at-point)
+		 (navi2ch-article-mode . navi2ch-article-url-at-point)
+		 (navi2ch-popup-article-mode
+		  . navi2ch-popup-article-url-at-point)
+		 ,@(mapcar (lambda (mode)
+			     (cons (intern (format "navi2ch-%s-mode"
+						   (car mode)))
+				   #'navi2ch-bm-url-at-point))
+			   navi2ch-bm-board-type-alist))))
+    (funcall (or (cdr (assq major-mode alist)) #'ignore) point)))
+
+(defun navi2ch-show-url-at-point (point)
+  "POINT $B$N2<$N%j%s%/$r;X$9(B URL $B$rI=<($7!"(Bkill-ring $B$K%3%T!<$9$k!#(B"
+  (interactive "d")
+  (let ((url (navi2ch-article-url-at-point point)))
+    (when url
+      (kill-new url)
+      (message "%s" url))))
+
 (eval-when-compile
   (mapatoms (lambda (symbol)
 	      (if (and (fboundp symbol)
